package db

import (
	"CokePlate/config"

	"sync"

	"github.com/prologic/bitcask"
)

type Backend interface {
	Sync() error
	Close() error
	Has(key []byte) bool
	Get(key []byte) ([]byte, error)
	Put(key, value []byte) error
	Delete(key []byte) error
}

var General *Database

type Database struct {
	backend Backend
	/*
		bitcask and most other backends have their own mutexing
		we implement our own here for any operations that are done with
		cached in-memory versions of our datasets
	*/
	mu *sync.RWMutex
}

// Initialize starts our database engine
func Initialize() {
	var casket *bitcask.Bitcask
	var err error
	if casket, err = bitcask.Open(config.Opt["data"]["directory"].(string) + "/" + "data"); err != nil {
		panic(err)
	}
	General = &Database{
		backend: casket,
		mu:      &sync.RWMutex{},
	}
}

// MustSync asserts a sync command to our underlying bitcask stores or else it will panic
func (d *Database) MustSync() {
	if err := d.backend.Sync(); err != nil {
		panic(err)
	}
}

// Replace deletes the old value at key and replaces it with the provided parameters (meant for json buckets)
func (d *Database) Replace(key string, value []byte) error {
	if err := d.backend.Delete([]byte(key)); err != nil {
		return err
	}
	if err := d.backend.Put([]byte(key), value); err != nil {
		return err
	}
	return nil
}
